import 'dart:convert';

int toInt(String string) {
  return int.tryParse(string) ?? 0;
}

class RawObj {
  final Object? data;

  RawObj(this.data);

  static RawObj? nullable(Object? data) {
    if (data == null) return null;
    return RawObj(data);
  }

  static RawMap toRawMap(Map<String, Object?> data) {
    return RawMap(data.map((k, v) => MapEntry(k, RawObj(v))));
  }

  static Map<String, RawObj?> toNullableRawMap(Map<String, Object?> data) {
    return data.map((k, v) => MapEntry(k, RawObj.nullable(v)));
  }

  bool toBool() {
    var obj = data;
    if (obj == null) return false;
    if (obj is bool) return obj;
    if (obj is int || obj is double) return obj != 0;
    if (obj is String) return obj.isNotEmpty;
    if (obj is Iterable) return obj.isNotEmpty;
    if (obj is Map) return obj.isNotEmpty;
    return false;
  }

  int toInt() {
    var obj = data;
    if (obj is int) return obj;
    if (obj is String) return int.tryParse(obj) ?? 0;
    if (toBool()) return 1;
    return 0;
  }

  @override
  String toString() {
    return data.toString();
  }

  bool? toNullBool() {
    if (data == null) return null;
    return toBool();
  }

  int? toNullInt() {
    if (data == null) return null;
    return toInt();
  }

  String? toNullString() {
    if (data == null) return null;
    return toString();
  }

  RawMap toMap() {
    var obj = data;
    if (obj is String) {
      try {
        obj = jsonDecode(obj);
      } catch (_) {}
    }
    if (obj is List<Object?>) {
      var i = 0;
      obj = Map.fromEntries(obj.map((e) => MapEntry((i++).toString(), e)));
    }
    if (obj is Map<String, Object?>) return toRawMap(obj);
    try {
      return toRawMap(Map<String, Object?>.from(obj as dynamic));
    } catch (_) {
      return RawMap({});
    }
  }

  Iterable<RawObj> toItrable() {
    var obj = data;
    if (obj is String) {
      try {
        obj = jsonDecode(obj);
      } catch (_) {}
    }
    if (obj is List) return obj.map((x) => RawObj(x));
    return [];
  }
}

class RawMap {
  Map<String, RawObj> map;
  RawMap(this.map);
  RawObj operator [](String key) {
    return map[key] ?? RawObj(null);
  }
}
